@@ -1,5 +1,5 @@
 /***************************************************************************
-* Copyright (c) 2016, Johan Mabille and Sylvain Corlay                     *
+* Copyright (c) 2016, Johan Mabille, Sylvain Corlay and Wolf Vollprecht    *
 *                                                                          *
 * Distributed under the terms of the BSD 3-Clause License.                 *
 *                                                                          *
@@ -30,7 +30,7 @@ namespace xt
     struct xcontainer_inner_types<xindexview<CT, I>>
     {
         using xexpression_type = std::decay_t<CT>;
-        using temporary_type = xarray<typename xexpression_type::value_type>;
+        using temporary_type = xarray<typename xexpression_type::value_type, xexpression_type::static_layout>;
     };
 
     template <class CT, class I>
@@ -39,10 +39,10 @@ namespace xt
         using inner_shape_type = std::array<std::size_t, 1>;
         using const_stepper = xindexed_stepper<xindexview<CT, I>>;
         using stepper = xindexed_stepper<xindexview<CT, I>, false>;
-        using const_broadcast_iterator = xiterator<const_stepper, inner_shape_type*>;
-        using broadcast_iterator = xiterator<stepper, inner_shape_type*>;
-        using const_iterator = const_broadcast_iterator;
-        using iterator = broadcast_iterator;
+        using const_iterator = xiterator<const_stepper, inner_shape_type*, DEFAULT_LAYOUT>;
+        using iterator = xiterator<stepper, inner_shape_type*, DEFAULT_LAYOUT>;
+        using const_reverse_iterator = std::reverse_iterator<const_iterator>;
+        using reverse_iterator = std::reverse_iterator<iterator>;
     };
 
     /**************
@@ -67,7 +67,6 @@ namespace xt
     class xindexview : public xview_semantic<xindexview<CT, I>>,
                        public xexpression_iterable<xindexview<CT, I>>
     {
-
     public:
 
         using self_type = xindexview<CT, I>;
@@ -86,22 +85,18 @@ namespace xt
         using inner_shape_type = typename iterable_base::inner_shape_type;
         using shape_type = inner_shape_type;
         using strides_type = shape_type;
-        using closure_type = const self_type;
 
         using indices_type = I;
 
         using stepper = typename iterable_base::stepper;
         using const_stepper = typename iterable_base::const_stepper;
 
-        using broadcast_iterator = typename iterable_base::broadcast_iterator;
-        using const_broadcast_iterator = typename iterable_base::const_broadcast_iterator;
-
-        using iterator = typename iterable_base::iterator;
-        using const_iterator = typename iterable_base::const_iterator;
-
         using temporary_type = typename xcontainer_inner_types<self_type>::temporary_type;
         using base_index_type = xindex_type_t<shape_type>;
 
+        static constexpr layout_type static_layout = layout_type::dynamic;
+        static constexpr bool contiguous_layout = false;
+
         template <class I2>
         xindexview(CT e, I2&& indices) noexcept;
 
@@ -114,6 +109,7 @@ namespace xt
         size_type size() const noexcept;
         size_type dimension() const noexcept;
         const inner_shape_type& shape() const noexcept;
+        layout_type layout() const noexcept;
 
         reference operator()();
         template <class... Args>
@@ -142,12 +138,12 @@ namespace xt
         template <class ST>
         stepper stepper_begin(const ST& shape);
         template <class ST>
-        stepper stepper_end(const ST& shape);
+        stepper stepper_end(const ST& shape, layout_type);
 
         template <class ST>
         const_stepper stepper_begin(const ST& shape) const;
         template <class ST>
-        const_stepper stepper_end(const ST& shape) const;
+        const_stepper stepper_end(const ST& shape, layout_type) const;
 
     private:
 
@@ -155,7 +151,7 @@ namespace xt
         const indices_type m_indices;
         const inner_shape_type m_shape;
 
-        void assign_temporary_impl(temporary_type& tmp);
+        void assign_temporary_impl(temporary_type&& tmp);
 
         friend class xview_semantic<xindexview<CT, I>>;
     };
@@ -185,7 +181,6 @@ namespace xt
     template <class ECT, class CCT>
     class xfiltration
     {
-
     public:
 
         using self_type = xfiltration<ECT, CCT>;
@@ -265,7 +260,7 @@ namespace xt
     }
 
     template <class CT, class I>
-    inline void xindexview<CT, I>::assign_temporary_impl(temporary_type& tmp)
+    inline void xindexview<CT, I>::assign_temporary_impl(temporary_type&& tmp)
     {
         std::copy(tmp.cbegin(), tmp.cend(), this->xbegin());
     }
@@ -300,6 +295,13 @@ namespace xt
     {
         return m_shape;
     }
+
+    template <class CT, class I>
+    inline layout_type xindexview<CT, I>::layout() const noexcept
+    {
+        return static_layout;
+    }
+
     //@}
 
     /**
@@ -423,7 +425,7 @@ namespace xt
 
     template <class CT, class I>
     template <class ST>
-    inline auto xindexview<CT, I>::stepper_end(const ST& shape) -> stepper
+    inline auto xindexview<CT, I>::stepper_end(const ST& shape, layout_type) -> stepper
     {
         size_type offset = shape.size() - dimension();
         return stepper(this, offset, true);
@@ -439,7 +441,7 @@ namespace xt
 
     template <class CT, class I>
     template <class ST>
-    inline auto xindexview<CT, I>::stepper_end(const ST& shape) const -> const_stepper
+    inline auto xindexview<CT, I>::stepper_end(const ST& shape, layout_type) const -> const_stepper
     {
         size_type offset = shape.size() - dimension();
         return const_stepper(this, offset, true);
